package gonganxitong

import (
	"path"
	"strings"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/aaa"
	"shylinux.com/x/icebergs/base/ctx"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/nfs"
	"shylinux.com/x/icebergs/base/tcp"
	"shylinux.com/x/icebergs/base/web"
	"shylinux.com/x/icebergs/core/code"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/api"
	"shylinux.com/x/community/src/gonganxitong/model"
)

type service struct {
	Table
	order   string `data:"904"`
	update  string `name:"update index name icon"`
	autogen string `name:"autogen name* help* class* table* dream" icon:"bi bi-plus-square-dotted"`
	oauth   string `name:"oauth" icon:"bi bi-shield-fill-check"`
	open    string `name:"open" role:"void"`
	conf    string `name:"conf" role:"void"`
}

func ServiceUID(m *ice.Message) string {
	return kit.Hashs(ice.Info.Hostname, ice.Info.Pwd, m.Prefix(web.PORTAL))
}
func (s service) Inputs(m *ice.Message, arg ...string) {
	switch s.Table.Inputs(m, arg...); arg[0] {
	case web.DREAM:
		m.Cut(arg[0])
	}
}
func (s service) Update(m *ice.Message, arg ...string) {
	uid := ServiceUID(m)
	if s.Select(m.Spawn(), model.UID, uid).Length() == 0 {
		s.Insert(m, kit.Simple(model.UID, uid, model.INDEX, m.Prefix(web.PORTAL), tcp.NODETYPE, ice.Info.NodeType, tcp.NODENAME, ice.Info.NodeName, nfs.PATHNAME, ice.Info.Pwd, tcp.HOSTNAME, ice.Info.Hostname)...)
	}
	s.Table.Update(m, kit.Dict(arg, m.OptionSimple(mdb.NAME, mdb.ICON), nfs.MODULE, ice.Info.Make.Module, nfs.VERSION, ice.Info.Make.Versions()), model.UID, uid)
	m.Cmd(s.PrefixPortal(m), Portal{}.CommandUpdate, uid)
}
func (s service) Oauth(m *ice.Message, arg ...string) {
	m.Options(m.Cmd(web.CHAT_WX_ACCESS).AppendSimple())
	m.Option("sess_cmd", kit.JoinWord(m.Prefix(aaa.SESS), mdb.CREATE))
	m.Option("user_cmd", kit.JoinWord(m.Prefix(aaa.USER), mdb.CREATE))
	m.Cmd(web.SPACE, ice.OPS, ctx.CONFIG, web.CHAT_WX_AGENT, web.SPACE, m.Option(ice.MSG_USERPOD))
	m.Cmd(web.CHAT_WX_ACCESS, web.OAUTH)
	m.Cmd(grant{}, web.HEADER)
	m.ProcessHold()
}
func (s service) Autogen(m *ice.Message, arg ...string) {
	p, table := path.Join(nfs.SRC, m.Option(mdb.NAME)), m.Option(mdb.TABLE)
	m.Option(nfs.MODULE, m.Cmdx(web.SPACE, m.Option(web.DREAM), code.AUTOGEN, code.IMPORT, m.Option(mdb.NAME)))
	m.Option("CLASS", strings.ToUpper(m.Option(mdb.CLASS)))
	m.Option("Class", kit.Capital(m.Option(mdb.CLASS)))
	m.Option("class", kit.LowerCapital(m.Option(mdb.CLASS)))
	m.Option("TABLE", strings.ToUpper(table))
	m.Option("Table", kit.Capital(table))
	m.Option("table", kit.LowerCapital(table))
	m.Option("what", kit.LowerCapital(table))
	msg := m.Cmd(api.GONGANXITONG_SERVICE)
	cmd := func(arg ...string) { m.Cmd(web.SPACE, m.Option(web.DREAM), arg) }
	if m.Option(web.DREAM) != "" {
		option := func(ctx, mod string) { cmd(code.AUTOGEN, code.IMPORT, mod, ctx, nfs.SRC_OPTION_GO) }
		option("shylinux.com/x/mysql-story", "db/mysql")
		option("shylinux.com/x/mysql-story", "db/sqlite")
		option("shylinux.com/x/community", "gonganxitong")
		option("shylinux.com/x/community", "renzhengshouquan")
		demo := func(p string) { cmd(nfs.SAVE, p, m.Cmdx(nfs.CAT, p)) }
		demo("src/template/web.code.autogen/demo.go")
		demo("src/template/web.code.go/demo.go")
		demo("src/template/web.code.js/demo.js")
	}
	save := func(name string, arg ...string) {
		cmd(nfs.DEFS, path.Join(p, name), msg.Template("template/"+kit.Select(name, arg, 0)))
	}
	save("model/model.go")
	save("common.go")
	save("portal.go")
	save("portal.json")
	save("user"+m.Option("Class")+".go", "userClass.go")
	save(m.Option("class")+".go", "class.go")
	kit.For(kit.Split(m.Option(mdb.TABLE)), func(table string) {
		msg.Option(mdb.TABLE, kit.LowerCapital(table))
		save(table+".go", "homework.go")
		save(table+".js", "homework.js")
	})
}
func (s service) Compile(m *ice.Message, arg ...string) {
	m.Cmdy(code.VIMER, code.COMPILE)
}
func (s service) List(m *ice.Message, arg ...string) {
	if len(arg) < 2 {
		s.Orders(m, s.Desc(model.UPDATED_AT)).Select(m)
	} else {
		s.SelectDetail(m, model.UID, arg[1])
	}
	if m.IsTech() {
		m.PushAction(s.Open, s.Conf, s.Code, s.Data).Action(s.Oauth, s.Autogen)
	} else {
		m.PushAction(s.Open).Action()
	}
	m.RenameAppend(model.NODENAME, model.SPACE, model.NODETYPE, model.SERVICE_TYPE, mdb.STATUS, model.SERVICE_STATUS)
	s.DisplayBase(m, "")
}
func (s service) Open(m *ice.Message, arg ...string) {
	s.spaceOpen(m, m.Option(ctx.INDEX))
}
func (s service) Conf(m *ice.Message, arg ...string) {
	s.spaceOpen(m, m.Option(ctx.INDEX), mdb.VIEW, mdb.TABLE)
}
func (s service) Code(m *ice.Message, arg ...string) {
	s.spaceOpen(m, code.VIMER, nfs.PATH, nfs.SRC, nfs.FILE, s.name(m)+"/portal.go")
}
func (s service) Data(m *ice.Message, arg ...string) {
	s.spaceOpen(m, web.CODE_MYSQL_QUERY, aaa.SESS, "mysql", mdb.DATABASE, s.name(m))
}

func init() { ice.TeamCtxCmd(service{}) }

func (s service) name(m *ice.Message) string {
	return kit.Select("", strings.Split(m.Option(ctx.INDEX), "."), -2)
}
func (s service) spaceOpen(m *ice.Message, index string, arg ...ice.Any) {
	m.ProcessOpen(kit.MergeURL(web.S(m.Option(web.SPACE))+web.C(index), arg...))
}
